<?php

namespace Espo\Modules\RealEstate\Services;

use \Espo\Core\Exceptions\Forbidden;
use \Espo\Core\Exceptions\BadRequest;
use \Espo\Core\Exceptions\NotFound;

class RealEstateSendMatches extends \Espo\Core\Templates\Services\Base
{

    protected function init()
    {
        $this->dependencies[] = 'container';
    }

    protected function getServiceFactory()
    {
        return $this->injections['serviceFactory'];
    }

    protected function getPreferences()
    {
        return $this->injections['preferences'];
    }

    protected function getConfig()
    {
        return $this->injections['config'];
    }

    protected function getMailSender()
    {
        return $this->getInjection('container')->get('mailSender');
    }

    protected function getSmptParams()
    {
        return $this->getPreferences()->getSmtpParams();
    }

    public function sendMatchesEmail($data)
    {
        if (empty($data['requestId']) || empty($data['propertyId'])) {
            throw new NotFound();
        }
        $request = $this->getEntityManager()->getEntity('RealEstateRequest', $data['requestId']);
        $property = $this->getEntityManager()->getEntity('RealEstateProperty', $data['propertyId']);

        if (!$request || !$property) {
            throw new NotFound();
        }

        $templateId = $this->getConfig()->get('realEstatePropertyTemplateId');
        if (empty($templateId)) {
            $GLOBALS['log']->error('RealEstate EmailSending[' . $request->id . ']: No Template in config');
            return false;
        }

        $emailBody = '';

        $requestService = $this->getServiceFactory()->create($request->getEntityName());
        $requestService->loadAdditionalFields($request);

        $propertyService = $this->getServiceFactory()->create($property->getEntityName());
        $propertyService->loadAdditionalFields($property);

        $contactId = $request->get('contactId');
        if (!$contactId) {
            $GLOBALS['log']->error('RealEstate EmailSending[' . $request->id . ']: No Contact in Request ');
            return false;
        }
        $contact = $this->getEntityManager()->getEntity('Contact', $contactId);
        if (!$contact) {
            $GLOBALS['log']->error('RealEstate EmailSending[' . $request->id . ']: Contact not found');
            return false;
        }
        if (!$contact->get('emailAddress')) {
            $GLOBALS['log']->error('RealEstate EmailSending[' . $request->id . ']: Contact has no email address');
            return false;
        }
        $toEmail = $contact->get('emailAddress');
        $ccEmailAddress = false;

        $entityHash = [];
        $entityHash[$request->getEntityName()] = $request;
        $entityHash[$property->getEntityName()] = $property;

        if ($this->getConfig()->get('realEstateEmailSendingAssignedUserCc') && $request->get('assignedUserId')) {
            $assignedUser = $this->getEntityManager()->getEntity('User', $request->get('assignedUserId'));
            $entityHash['User'] = $assignedUser;
            if ($assignedUser->get('emailAddress')) {
                $ccEmailAddress = $assignedUser->get('emailAddress');
            }
        }
        $entityHash['Person'] = $contact;

        $emailTemplateParams = array(
            'entityHash' => $entityHash,
            'emailAddress' => $toEmail,
        );
        if ($request->hasField('parentId') && $request->hasField('parentType')) {
            $emailTemplateParams['parentId'] = $request->get('parentId');
            $emailTemplateParams['parentType'] = $request->get('parentType');
        }

        $emailTemplateService = $this->getServiceFactory()->create('EmailTemplate');
        $emailTemplate = $emailTemplateService->parse($templateId, $emailTemplateParams, true);
        $emailBody = $emailTemplate['body'];

        $emailData = array(
            'to' => $toEmail,
            'subject' => $emailTemplate['subject'],
            'body' => $emailBody,
            'isHtml' => $emailTemplate['isHtml'],
            'parentId' => $request->id,
            'parentType' => $request->getEntityName(),
        );
        if ($ccEmailAddress) {
            $emailData['cc'] = $ccEmailAddress;
        }

        $attachmentList = [];

        foreach ($emailTemplate['attachmentsIds'] as $attachmentId) {
            $attachment = $this->getEntityManager()->getEntity('Attachment', $attachmentId);
            if ($attachment) {
                $attachmentList[] = $attachment;
            }
        }

        foreach ($property->getLinkMultipleIdList('images') as $attachmentId) {
            $attachment = $this->getEntityManager()->getEntity('Attachment', $attachmentId);
            if ($attachment) {
                $attachmentList[] = $attachment;
            }
        }

        $email = $this->getEntityManager()->getEntity('Email');
        $email->set($emailData);

        $emailSender = $this->getMailSender();
        if ($this->getSmptParams()) {
            $emailSender->useSmtp($this->getSmptParams());
        }

        $message = new \Zend\Mail\Message();
        $emailSender->send($email, array(), $message, $attachmentList);
    }
}
